
angular.module('beamng.apps')
.directive('steinTpms', ['$interval', function($interval) {
  return {
    templateUrl: '/ui/modules/apps/SteinTPMS/app.html',
    replace: true,
    restrict: 'EA',
    link: function (scope, element, attrs) {
    	//console.log("loaded");
        var incompatibleTPMS = false;
        var showTimer = 0;
        var checkVehicle = true;

        var updThread;
        /*var indLeft = loadImage("indicator_left.png");
        var indRight = loadImage("indicator_right.png");
        var lowBeam = loadImage("low_beams.png");
        var highBeam = loadImage("high_beams.png");
        var fuelIconOff = loadImage("fuel-icon-off.png");
        var fuelIconOn = loadImage("fuel-icon-on.png");
        var brakeIcon = loadImage("brake.png");*/

        function loadImage(url) {
            var img = new Image();
            img.src = "/ui/modules/apps/SteinTPMS/" + url;
            return img;
        }

        var gearType = "automaticGearbox";

        var streams = ['tirePressureData'];
        StreamsManager.add(streams);
        scope.$on('$destroy', function () {
            StreamsManager.remove(streams);
            $interval.cancel(updThread);
        });
        scope.currentTheme = {
            background: 'rgba(255, 255, 255, 0.9)',
            text: 'rgba(0, 0, 0, 0.9)',
            border: 'rgba(0, 0, 0, 0.9)',

            engIdle: 'rgba(160, 160, 255, 0.5)',
            engRedline: 'rgba(255, 0, 0, 0.5)',
            engNormal: 'rgba(0, 255, 0, 0.5)',
        };

        var wheelData = [];
        var c = element.find("canvas")[0];
        CanvasRenderingContext2D.prototype.roundRect = function (x, y, w, h, r) {
            if (w < 2 * r) r = w / 2;
            if (h < 2 * r) r = h / 2;
            this.beginPath();
            this.moveTo(x+r, y);
            this.arcTo(x+w, y,   x+w, y+h, r);
            this.arcTo(x+w, y+h, x,   y+h, r);
            this.arcTo(x,   y+h, x,   y,   r);
            this.arcTo(x,   y,   x+w, y,   r);
            this.closePath();
            return this;
        }

        c.width = 100; 
        c.height = 140;
        var ctx = c.getContext('2d');
        ctx.imageSmoothingEnabled = false;

        var gAlpha = 0; //fading alpha
        var font = "ARLRDBD";

        if(document.getElementById("sTPMS")) {
        	document.getElementById("sTPMS").style.opacity = 1;
		}
        updThread = $interval(function() {
            if(incompatibleTPMS) {
                if(showTimer == 2) {
                	if(document.getElementById("sTPMS").style.opacity == 1) {
	                	//console.log("fade out");
	                    document.getElementById("sTPMS").style.opacity = 0;
	                }
                } else {
                    showTimer++;
                }
            } else {
                showTimer = 0;
                if(document.getElementById("sTPMS").style.opacity == 0) {
	                //console.log("fade in");
	                document.getElementById("sTPMS").style.opacity = 1;
	            }
            }
        }, 1000);

        scope.$on('streamsUpdate', function (event, data) {
        	if(data.tirePressureData) {
	        	wheelData = [];
	            for(var wheel in data.tirePressureData) {
	                // Mod for one decimal place
					var tirePressure = Math.round(UiUnits.pressure(data.tirePressureData[wheel]).val * 10) / 10;
	                wheelData.push({
	                    name: wheel,
	                    pressure: tirePressure
	                });
	            }

	            if(checkVehicle) {
	                
	                if(wheelData.length < 5 && wheelData.length > 2) {
	                    incompatibleTPMS = false;
	                } else {
	                    incompatibleTPMS = true;
	                }
	                checkVehicle = false;

	                //console.log("Checked vehicle -- incompatible: " + incompatibleTPMS);
	            }
	        }
            //if(document.getElementById("sTPMS").style.opacity == 0) return;

            ctx.clearRect(0, 0, c.width, c.height);

            ctx.fillStyle = "rgba(0, 0, 0, 0.5)";
            ctx.roundRect(0, 0, c.width, c.height, 5).fill();

            var padding = 5;
            if(wheelData.length == 3) {
                var fwheel = new Tire("F", c.width / 2, 20 + padding);
                fwheel.draw();

                var rlwheel = new Tire("RL", 12 + padding, c.height - 20 -  padding);
                rlwheel.draw();

                var rrwheel = new Tire("RR", c.width - 12 - padding, c.height - 20 - padding);
                rrwheel.draw();
            } else if(wheelData.length == 4) {

                var flwheel = new Tire("FL", 12 + padding, 20 + padding);
                flwheel.draw();

                var frwheel = new Tire("FR", c.width - 12 - padding, 20 + padding);
                frwheel.draw();

                var rlwheel = new Tire("RL", 12 + padding, c.height - 20 - padding);
                rlwheel.draw();

                var rrwheel = new Tire("RR", c.width - 12 - padding, c.height - 20 - padding);
                rrwheel.draw();
            }

            ctx.textAlign = "center";
            ctx.textBaseline = "middle";
            ctx.font = "16px " + font;
            ctx.fillStyle = "white";

            if(!incompatibleTPMS) {
                //ctx.fillText("Pressure", c.width / 2, c.height / 2);
				ctx.fillText("(" + UiUnits.pressure().unit + ")", c.width / 2, c.height / 2);
				ctx.textAlign = "middle";
				// Mod to display Front and Rear
				ctx.fillText("Front", c.width / 2, c.height / 3);
				ctx.fillText("Rear", c.width / 2, 2 * c.height / 3);
            } else {
                ctx.font = "14px " + font;
                ctx.fillText("TPMS Error", c.width / 2, c.height / 2 - 30);
                ctx.fillText("TOO MANY", c.width / 2, c.height / 2 + 10);
                ctx.fillText("TIRES", c.width / 2, c.height / 2 + 30);
            }
        });

        scope.$on('VehicleFocusChanged', function (_, data) {
            checkVehicle = true;
        });


        var Tire = function(name, x, y) {
            this.name = name;
            this.x = x;
            this.y = y;
        }
        Tire.prototype.draw = function() {
            if(!getTireData(this.name)) return;
            ctx.fillStyle = "rgba(0, 0, 0, 0.5)";
            // Mod for Bar compatibility
			if(getTireData(this.name).pressure <= 0.3) {
                ctx.fillStyle = "rgba(255, 0, 0, 0.7)";
            }
            ctx.strokeStyle = "rgba(0, 0, 0, 0.5)";
            var tire = ctx.roundRect(this.x - 24 / 2, this.y - 40 / 2, 24, 40, 5); //centered
            ctx.lineWidth = 2;
            tire.fill(); tire.stroke();
            
            ctx.textAlign = "center";
            ctx.textBaseline = "middle";
            ctx.font = "16px " + font;
            ctx.fillStyle = "white";
            ctx.fillText(getTireData(this.name).pressure, this.x, this.y);
        };

        function getTireData(name) {
            for(var i in wheelData) {
                if(wheelData[i].name == name) return wheelData[i];
            }
            return {};
        }
        function getWidth(txt) {
            return ctx.measureText(txt).width;
        }

        function pad( number, width ){
            width -= number.toString().length;
            if ( width > 0 ){
                return new Array( width + (/\./.test( number ) ? 2 : 1) ).join( '0' ) + number;
            }
            return number + ""; // always return a string
        }

        function getDefaultValue(type, defVal) {
            return localStorage.getItem("classicUI_" + type) || defVal;
        }
    }
  };
}]);
